Index: src/Fl_lock.cxx
===================================================================
--- src/Fl_lock.cxx	(revision 5555)
+++ src/Fl_lock.cxx	(working copy)
@@ -28,6 +28,8 @@
 
 #include <FL/Fl.H>
 #include <config.h>
+#include <stdlib.h>
+#include <string.h>
 
 /*
    From Bill:
@@ -205,6 +207,57 @@
 
 #endif
 
+
+////////////////////////////////////////////////////////////////
+// recall
+
+struct RcData {
+  Fl_Timeout_Handler callback;
+  void* data;
+};
+
+static size_t rc_array_size = 0; 
+static size_t rc_data_size = 0; 
+static RcData* rc_data = NULL;
+
+static void rc_check(void*)
+{
+  while(rc_data_size) {
+    /* allow recall to be called recursively inside callbacks */
+    RcData buf = *rc_data;
+    memmove(rc_data, rc_data + 1, sizeof(RcData) * --rc_data_size);
+    (*buf.callback)(buf.data);
+  }
+}
+
+void Fl::recall(Fl_Timeout_Handler callback, void* data)
+{
+  fl_lock_function();
+
+  if(rc_data_size + 1 > rc_array_size) {
+    if(!rc_array_size)
+    {
+      add_check(rc_check);
+      rc_array_size = 1;
+      rc_data = (RcData*)malloc(sizeof(RcData));
+    }
+    else
+    {
+      rc_array_size *= 2;
+      rc_data = (RcData*)realloc(rc_data, sizeof(RcData) * rc_array_size);
+    }
+    if(!rc_data)
+      fatal("cannot allocate enough memory for recall");
+  }
+
+  RcData* buf = rc_data + rc_data_size++;
+  buf->callback = callback;
+  buf->data = data;
+
+  awake();
+  fl_unlock_function();
+}
+
 //
 // End of "$Id$".
 //
Index: FL/Fl.H
===================================================================
--- FL/Fl.H	(revision 5555)
+++ FL/Fl.H	(working copy)
@@ -266,6 +266,7 @@
   static void lock();
   static void unlock();
   static void awake(void* message = 0);
+  static void recall(Fl_Timeout_Handler, void* = 0);
   static void* thread_message();
 
   // Widget deletion:
Index: documentation/Fl.html
===================================================================
--- documentation/Fl.html	(revision 5555)
+++ documentation/Fl.html	(working copy)
@@ -109,6 +109,7 @@
 	<LI><A HREF="#Fl.pushed">pushed</A></LI>
 	<LI><A HREF="#Fl.readqueue">readqueue</A></LI>
 	<LI><A HREF="#Fl.ready">ready</A></LI>
+	<LI><A HREF="#Fl.recall">recall</A></LI>
 	<LI><A HREF="#Fl.redraw">redraw</A></LI>
 	<LI><A HREF="#Fl.release">release</A></LI>
 	<LI><A HREF="#Fl.remove_check">remove_check</A></LI>
@@ -1035,6 +1036,35 @@
 }
 </PRE></UL>
 
+<H4><A NAME="Fl.recall">void recall(Fl_Timeout_Handler, void* = 0);</A></H4>
+
+<P><TT>recall()</TT> executes the specified handler in the main thread as
+soon as possible. Multiple and recursive calls to <TT>recall()</TT> are
+allowed. Handlers are FIFO scheduled. <TT>recall()</TT> overwrites the message
+pointer returned by <A HREF="#Fl.thread_message"><TT>thread_message()</TT></A>,
+thus never mix <TT>recall()</TT> and <A HREF="#Fl.awake"><TT>awake()</TT></A>
+based code. Extra calls to <TT>awake()</TT> are still permissible, however,
+to break the main execution loop.
+
+<P>Sample code:
+
+<UL><PRE>
+done(void*) {
+  fl_alert("Finished processing");
+}
+
+thread() {
+  /* processing */
+  recall(done);
+}
+
+main() {
+  Fl::lock();
+  /* run thread */
+  return Fl::run();
+}
+</PRE></UL>
+
 <H4><A NAME="Fl.redraw">void redraw();</A></H4>
 
 <P>Redraws all widgets.
